/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: OSGiDroolsWorkingMemory.java 1741 2008-07-21 15:13:28Z jlegrand $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.osgi.impl;

import java.util.Collection;
import java.util.Collections;
import java.util.Map;
import java.util.logging.Logger;

import org.drools.RuleBase;
import org.drools.RuleBaseConfiguration;
import org.drools.RuleBaseFactory;
import org.drools.compiler.PackageBuilder;
import org.drools.compiler.PackageBuilderConfiguration;
import org.drools.rule.builder.dialect.java.JavaDialectConfiguration;
import org.osgi.framework.ServiceReference;
import org.ow2.jasmine.rules.AbstractDroolsWorkingMemory;
import org.ow2.jasmine.rules.osgi.api.IOSGiDroolsWorkingMemory;
import org.ow2.jasmine.rules.osgi.api.IPackageAdder;

/**
 * A service implementation for OSGi to manage the Drools working memory.
 *
 * @author Loris Bouzonnet
 */
public class OSGiDroolsWorkingMemory extends AbstractDroolsWorkingMemory implements IOSGiDroolsWorkingMemory {

    /**
     * Logger.
     */
    private static Logger logger = Logger
            .getLogger(OSGiDroolsWorkingMemory.class.getName());

    /**
     * Delegate class loading to a set of bundles.
     */
    private final MultiBundleClassLoader multiBundleClassLoader;

    /**
     * Used package adders.
     */
    protected final Map<ServiceReference, IPackageAdder> packageAdders;

    // ------------------------------------------------------------------------
    // Constructor
    // ------------------------------------------------------------------------

    /**
     * Create a manager of working memory that use OSGi to add (and remove)
     * packages.
     *
     * @param externalLoader
     *            a classloader to use for adding package
     * @param packageAdders
     *            initial package adders
     */
    public OSGiDroolsWorkingMemory(final ClassLoader externalLoader,
            final Map<ServiceReference, IPackageAdder> packageAdders) {

        super(externalLoader, logger);
        this.packageAdders = packageAdders;
        multiBundleClassLoader = new MultiBundleClassLoader(getClass()
                .getClassLoader());
    }

    // ------------------------------------------------------------------------
    // Implementation of the IDroolsWorkingMemory interface
    // ------------------------------------------------------------------------


    public void start() throws Exception {
        // Check if some rules packages are ready to be registered
        if (!packageAdders.isEmpty()) {
            addPackages(packageAdders.values());
        }
    }

    /**
     * Add a rules package by using the given package adder.
     *
     * @param serviceReference
     *            a reference to the given package adder
     * @param packageAdder
     *            a package adder
     * @throws Exception
     *             if a package cannot be built or added to the rule base
     */
    public void addPackage(final ServiceReference serviceReference,
            final IPackageAdder packageAdder) throws Exception {

        if (packageAdders.put(serviceReference, packageAdder) == null) {
            addPackages(Collections.singletonList(packageAdder));
        }
    }

    /**
     * Remove a package adder for a given service reference.
     *
     * @param serviceReference
     *            a reference to a given package adder
     * @throws Exception
     *             if the working memory cannot be restarted
     */
    public void removePackage(final ServiceReference serviceReference)
            throws Exception {
        multiBundleClassLoader.removeBundleClassLoader(packageAdders.get(
                serviceReference).getBundle());
        if (packageAdders.remove(serviceReference) != null
                && packageAdders.size() > 0) {
            addPackages(packageAdders.values());
        }
        if (packageAdders.size() == 0) {
            stop();
        }
    }

    // ------------------------------------------------------------------------
    // Private methods
    // ------------------------------------------------------------------------

    /**
     * Add some rules packages by using the given package adders.
     *
     * @param packageAdders
     *            a collection of package adders
     * @throws Exception
     *             if a package cannot be built or added to the rule base
     */
    private void addPackages(final Collection<IPackageAdder> packageAdders)
            throws Exception {
        ClassLoader lastClassLoader = Thread.currentThread()
                .getContextClassLoader();
        Thread.currentThread().setContextClassLoader(
                getClass().getClassLoader());
        try {
            PackageBuilderConfiguration pkgBuilderCfg = new PackageBuilderConfiguration(
                    classLoader);
            JavaDialectConfiguration javaConf = (JavaDialectConfiguration) pkgBuilderCfg
                    .getDialectConfiguration("java");
            javaConf.setCompiler(JavaDialectConfiguration.JANINO);
            if (dumpDir != null) {
                pkgBuilderCfg.setDumpDir(dumpDir);
            }
            RuleBase ruleBase = RuleBaseFactory
                    .newRuleBase(new RuleBaseConfiguration(
                            multiBundleClassLoader));
            for (IPackageAdder packageAd : packageAdders) {
                if (packageAd instanceof IPackageAdder) {
                    IPackageAdder osgiPackageAd = packageAd;

                    // Use a new package builder for each package adder, since
                    // bundles can define different packages
                    PackageBuilder packageBuilder = new PackageBuilder(
                            pkgBuilderCfg);
                    osgiPackageAd.addPackage(packageBuilder, ruleBase);
                    multiBundleClassLoader.addBundleClassLoader(osgiPackageAd
                            .getBundle());
                }
            }
            if (workingMemory != null) {
                restoreWorkingMemory(ruleBase);
            } else {
                // Create and init a new WorkingMemory
                createWorkingMemory(ruleBase);
            }
        } finally {
            Thread.currentThread().setContextClassLoader(lastClassLoader);
        }
    }

}
