/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.osgi.impl;

import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URL;
import java.util.Enumeration;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.drools.RuleBase;
import org.drools.compiler.PackageBuilder;
import org.drools.rule.Package;
import org.drools.rule.Rule;
import org.osgi.framework.Bundle;
import org.ow2.jasmine.rules.osgi.api.IPackageAdder;

/**
 * Implement a service to add {@link Package} into a {@link RuleBase}.
 * @author Loris Bouzonnet
 */
public class PackageAdder implements IPackageAdder {

    /**
     * Logger.
     */
    private static Logger logger =
        Logger.getLogger(PackageAdder.class.getName());

    /**
     * Bundle containing rules.
     */
    private final Bundle bundle;

    /**
     * Create a new package adder to add rules belonging to a given bundle into the working memory.
     * @param bundle bundle containing rules
     */
    public PackageAdder(final Bundle bundle) {
        this.bundle = bundle;
    }

    /**
     * Add a rules package into the providing {@link RuleBase}, by using the given {@link PackageBuilder}.
     * @param packageBuilder a package builder
     * @param ruleBase a rule base
     * @throws Exception if a package cannot be built or added to the rule base
     */
    public void addPackage(final PackageBuilder builder, final RuleBase ruleBase)
    throws Exception {

        // List all DRL files in the bundle
        Enumeration<?> drlPaths = bundle.findEntries("/", "*.drl", true);

        if(drlPaths != null) {
            while(drlPaths.hasMoreElements()) {
                URL drlPath = (URL) drlPaths.nextElement();

                // Search for a domain description file
                String externalForm = drlPath.toExternalForm();
                URL dslPath = bundle.getEntry(
                        externalForm.substring(0, externalForm.length() - 4) + ".dsl");

                Reader fileReader = new InputStreamReader(drlPath.openStream());
                if(dslPath == null) {
                    try {
                        builder.addPackageFromDrl(fileReader);
                        logger.log(Level.INFO, drlPath.getPath() + " loaded");
                    } finally {
                        fileReader.close();
                    }
                } else {
                    Reader dslReader = new InputStreamReader(dslPath.openStream());
                    try {
                        builder.addPackageFromDrl(fileReader, dslReader);
                        logger.log(Level.INFO, drlPath.getPath()
                                + " and " + dslPath.getPath() + " loaded");
                    } finally {
                        fileReader.close();
                        dslReader.close();
                    }
                }
            }
        }

        // List all DRL XML files in the bundle
        Enumeration<?> drlXmlPaths = bundle.findEntries("/", "*.drl.xml", true);

        if(drlXmlPaths != null) {
            while(drlXmlPaths.hasMoreElements()) {
                URL drlXmlPath = (URL) drlXmlPaths.nextElement();

                Reader fileReader = new InputStreamReader(drlXmlPath.openStream());
                try {
                    builder.addPackageFromXml(fileReader);
                    logger.log(Level.INFO, drlXmlPath.getPath() + " loaded");
                } finally {
                    fileReader.close();
                }
            }
        }

        Package pack = builder.getPackage();
        if(pack != null) {
            pack.checkValidity();
            Rule[] rules = pack.getRules();
            if(rules.length != 0) {
                ruleBase.addPackage(pack);
            }
        }
    }

    /**
     * @return the bundle containing rules
     */
    public Bundle getBundle() {
        return bundle;
    }

}
