/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: RulesActivator.java 1741 2008-07-21 15:13:28Z jlegrand $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.osgi.impl;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.framework.ServiceEvent;
import org.osgi.framework.ServiceListener;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.ServiceRegistration;
import org.ow2.jasmine.rules.api.IDroolsWorkingMemory;
import org.ow2.jasmine.rules.osgi.api.IPackageAdder;

/**
 * Activator that registers a package adder for this bundle, this defining rules.
 * Note that rules have to belong to a same package.
 * This class also provide the working memory to allow bundles adding some globals.
 * @author Loris Bouzonnet
 */
public abstract class RulesActivator implements BundleActivator {

    /**
     * A service registration for the package adder of this bundle.
     */
    private ServiceRegistration packageAdderRegistration;

    /**
     * Provide the working memory to allow bundles adding some globals into this.
     */
    private IDroolsWorkingMemory droolsWorkingMemory;

    /**
     * Listener to be notified when working memory is (un)registered.
     */
    private ServiceListener droolsWorkingMemoryListener;

    /**
     * Reference on working memory service.
     */
    private ServiceReference droolsWorkingMemoryReference;

    /**
     * True if the working memory is available.
     */
    private volatile boolean started = false;

    public final void start(final BundleContext context) throws Exception {

        droolsWorkingMemoryListener =
            new ServiceListener() {
            public void serviceChanged(final ServiceEvent event) {
                ServiceReference ref = event.getServiceReference();
                // React on REGISTERED|UNREGISTERING events
                switch(event.getType()) {
                case ServiceEvent.REGISTERED:
                    droolsWorkingMemory = (IDroolsWorkingMemory) context.getService(ref);

                    if(!started) {
                        // Perform the real start of the bundle
                        try {
                            doStart(context);
                        } catch (Exception e1) {
                            // TODO Auto-generated catch block
                            e1.printStackTrace();
                        }

                        // Register a package adder for this bundle
                        packageAdderRegistration =
                            context.registerService(IPackageAdder.class.getName(),
                                    new PackageAdder(context.getBundle()), null);
                        started = true;
                    }
                    break;
                case ServiceEvent.UNREGISTERING:
                    droolsWorkingMemory = null;

                    if(started) {
                        // Unregister the package adder
                        if(packageAdderRegistration != null) {
                            packageAdderRegistration.unregister();
                            packageAdderRegistration = null;
                        }

                        // Perform the real stop of the bundle
                        try {
                            doStop(context);
                        } catch (Exception e) {
                            // TODO Auto-generated catch block
                            e.printStackTrace();
                        }
                        started = false;
                    }
                    break;
                }
            }
        };
        try {
            context.addServiceListener(droolsWorkingMemoryListener,
                    "(objectClass=" + IDroolsWorkingMemory.class.getName() + ")");
        } catch (InvalidSyntaxException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        // Get the working memory if this is available
        droolsWorkingMemoryReference = context.getServiceReference(IDroolsWorkingMemory.class.getName());
        if(droolsWorkingMemoryReference != null) {
            droolsWorkingMemory = (IDroolsWorkingMemory) context.getService(droolsWorkingMemoryReference);
        }

        // Perform the real start of the bundle
        doStart(context);

        // Register a package adder for this bundle
        packageAdderRegistration =
            context.registerService(IPackageAdder.class.getName(),
                    new PackageAdder(context.getBundle()), null);

        started = true;
    }

    public final void stop(final BundleContext context) throws Exception {
        // Unregister the package adder
        if(packageAdderRegistration != null) {
            packageAdderRegistration.unregister();
            packageAdderRegistration = null;
        }

        // Perform the real stop of the bundle
        doStop(context);

        // Release the service reference on working memory
        if(droolsWorkingMemoryReference != null) {
            context.ungetService(droolsWorkingMemoryReference);
        }

        started = false;
    }

    /**
     * Perform the real start of the bundle.
     * @param context bundle context
     * @throws Exception if the bundle cannot be started
     */
    protected abstract void doStart(BundleContext context) throws Exception;

    /**
     * Perform the real stop of the bundle.
     * @param context bundle context
     * @throws Exception if the bundle cannot be stopped
     */
    protected abstract void doStop(BundleContext context) throws Exception;

    /**
     * @return working memory
     */
    public IDroolsWorkingMemory getDroolsWorkingMemory() {
        return droolsWorkingMemory;
    }

}
